@@ -55,6 +55,7 @@
 
 static GInetAddress *SSDP_V6_LL_ADDR = NULL;
 static GInetAddress *SSDP_V6_SL_ADDR = NULL;
+static GInetAddress *SSDP_V6_GL_ADDR = NULL;
 static GInetAddress *SSDP_V4_ADDR = NULL;
 
 static void
@@ -109,7 +110,8 @@ struct _GSSDPHeaderField {
 };
 typedef struct _GSSDPHeaderField GSSDPHeaderField;
 
-enum {
+enum
+{
         PROP_0,
         PROP_SERVER_ID,
         PROP_IFACE,
@@ -124,6 +126,7 @@ enum {
         PROP_BOOT_ID,
         PROP_CONFIG_ID,
         PROP_PORT,
+        PROP_HOST_ADDR,
 };
 
 enum {
@@ -327,6 +330,9 @@ gssdp_client_get_property (GObject    *object,
                 g_value_set_string (value,
                                     gssdp_client_get_host_ip (client));
                 break;
+        case PROP_HOST_ADDR:
+                g_value_set_object (value, gssdp_client_get_address (client));
+                break;
         case PROP_ACTIVE:
                 g_value_set_boolean (value, priv->active);
                 break;
@@ -376,14 +382,11 @@ gssdp_client_set_property (GObject      *object,
                 priv->device.network = g_value_dup_string (value);
                 break;
         case PROP_HOST_IP:
-                {
-                        const char *addr = g_value_get_string (value);
-                        if (addr != NULL) {
-                                priv->device.host_addr =
-                                    g_inet_address_new_from_string (addr);
-                        }
-                        break;
-                }
+                priv->device.host_ip = g_value_dup_string (value);
+                break;
+        case PROP_HOST_ADDR:
+                priv->device.host_addr = g_value_dup_object (value);
+                break;
         case PROP_HOST_MASK:
                 priv->device.host_mask = g_value_dup_object (value);
                 break;
@@ -494,9 +497,7 @@ gssdp_client_class_init (GSSDPClientClass *klass)
                           NULL,
                           G_PARAM_READWRITE |
                           G_PARAM_CONSTRUCT_ONLY |
-                          G_PARAM_STATIC_NAME |
-                          G_PARAM_STATIC_NICK |
-                          G_PARAM_STATIC_BLURB));
+                          G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:network:(attributes org.gtk.Property.get=gssdp_client_get_network):
@@ -508,55 +509,69 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          * the network. Otherwise, expect this to be the network IP address by
          * default.
          **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_NETWORK,
-                 g_param_spec_string
-                         ("network",
-                          "Network ID",
-                          "The network this client is currently connected to.",
-                          NULL,
-                          G_PARAM_READWRITE |
-                          G_PARAM_CONSTRUCT |
-                          G_PARAM_STATIC_NAME |
-                          G_PARAM_STATIC_NICK |
-                          G_PARAM_STATIC_BLURB));
+        g_object_class_install_property (
+                object_class,
+                PROP_NETWORK,
+                g_param_spec_string (
+                        "network",
+                        "Network ID",
+                        "The network this client is currently connected to.",
+                        NULL,
+                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:host-ip:(attributes org.gtk.Property.get=gssdp_client_get_host_ip):
          *
          * The IP address of the assoicated network interface.
+         *
+         * Deprecated: 1.6. Use [property@GSSDP.Client:address] instead.
          **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_HOST_IP,
-                 g_param_spec_string ("host-ip",
-                                      "Host IP",
-                                      "The IP address of the associated"
-                                      "network interface",
-                                      NULL,
-                                      G_PARAM_READWRITE |
-                                      G_PARAM_CONSTRUCT |
-                                      G_PARAM_STATIC_NAME |
-                                      G_PARAM_STATIC_NICK |
-                                      G_PARAM_STATIC_BLURB));
+        g_object_class_install_property (
+                object_class,
+                PROP_HOST_IP,
+                g_param_spec_string ("host-ip",
+                                     "Host IP",
+                                     "The IP address of the associated"
+                                     "network interface",
+                                     NULL,
+                                     G_PARAM_READWRITE |
+                                             G_PARAM_CONSTRUCT_ONLY |
+                                             G_PARAM_STATIC_STRINGS));
+
+        /**
+         * GSSDPClient:address:(attributes org.gtk.Property.get=gssdp_client_get_address):
+         *
+         * The network address this client is bound to.
+         * Since: 1.6
+         **/
+        g_object_class_install_property (
+                object_class,
+                PROP_HOST_ADDR,
+                g_param_spec_object ("address",
+                                     "Network address",
+                                     "The internet address of the client",
+                                     G_TYPE_INET_ADDRESS,
+                                     G_PARAM_READWRITE |
+                                             G_PARAM_CONSTRUCT_ONLY |
+                                             G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:host-mask:(attributes org.gtk.Property.get=gssdp_client_get_address_mask):
          *
          * The network mask of the assoicated network interface.
          **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_HOST_MASK,
-                 g_param_spec_object ("host-mask",
-                                      "Host network mask",
-                                      "The IP netmask of the associated"
-                                      "network interface",
-                                      G_TYPE_INET_ADDRESS_MASK,
-                                      G_PARAM_READWRITE |
-                                      G_PARAM_CONSTRUCT |
-                                      G_PARAM_STATIC_STRINGS));
+        g_object_class_install_property (
+                object_class,
+                PROP_HOST_MASK,
+                g_param_spec_object ("host-mask",
+                                     "Host network mask",
+                                     "The IP netmask of the associated"
+                                     "network interface",
+                                     G_TYPE_INET_ADDRESS_MASK,
+                                     G_PARAM_READWRITE |
+                                             G_PARAM_CONSTRUCT_ONLY |
+                                             G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:active:(attributes org.gtk.Property.get=gssdp_client_get_active):
@@ -566,18 +581,15 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          * not. In most cases, you don't want to touch this property.
          *
          **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_ACTIVE,
-                 g_param_spec_boolean
-                         ("active",
-                          "Active",
-                          "TRUE if the client is active.",
-                          TRUE,
-                          G_PARAM_READWRITE |
-                          G_PARAM_STATIC_NAME |
-                          G_PARAM_STATIC_NICK |
-                          G_PARAM_STATIC_BLURB));
+        g_object_class_install_property (
+                object_class,
+                PROP_ACTIVE,
+                g_param_spec_boolean ("active",
+                                      "Active",
+                                      "TRUE if the client is active.",
+                                      TRUE,
+                                      G_PARAM_READWRITE |
+                                              G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:socket-ttl:
@@ -586,18 +598,17 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          * If not set (or set to 0) the value recommended by UPnP will be used.
          * This property can only be set during object construction.
          */
-        g_object_class_install_property
-                (object_class,
-                 PROP_SOCKET_TTL,
-                 g_param_spec_uint
-                        ("socket-ttl",
-                         "Socket TTL",
-                         "Time To Live for client's sockets",
-                         0, 255,
-                         0,
-                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
-                         G_PARAM_STATIC_BLURB));
+        g_object_class_install_property (
+                object_class,
+                PROP_SOCKET_TTL,
+                g_param_spec_uint ("socket-ttl",
+                                   "Socket TTL",
+                                   "Time To Live for client's sockets",
+                                   0,
+                                   255,
+                                   0,
+                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                           G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:msearch-port:
@@ -606,20 +617,19 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          * network. If not set (or set to 0) a random port will be used.
          * This property can be only set during object construction.
          *
-         * Deprecated: 1.6.0: Use GSSDPClient:port instead
+         * Deprecated: 1.6.0: Use [property@GSSDP.Client:port] instead
          */
-        g_object_class_install_property
-                (object_class,
-                 PROP_MSEARCH_PORT,
-                 g_param_spec_uint
-                        ("msearch-port",
-                         "M-SEARCH port",
-                         "UDP port to use for M-SEARCH requests",
-                         0, G_MAXUINT16,
-                         0,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS));
+        g_object_class_install_property (
+                object_class,
+                PROP_MSEARCH_PORT,
+                g_param_spec_uint ("msearch-port",
+                                   "M-SEARCH port",
+                                   "UDP port to use for M-SEARCH requests",
+                                   0,
+                                   G_MAXUINT16,
+                                   0,
+                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                           G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:port:
@@ -628,20 +638,19 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          * network. If not set (or set to 0) a random port will be used.
          * This property can be only set during object construction.
          *
-         * Deprecated: 1.6.0
+         * Since: 1.6.0
          */
-        g_object_class_install_property
-                (object_class,
-                 PROP_PORT,
-                 g_param_spec_uint
-                 ("port",
-                  "M-SEARCH port",
-                  "UDP port to use for M-SEARCH requests",
-                  0, G_MAXUINT16,
-                  0,
-                  G_PARAM_READWRITE |
-                          G_PARAM_CONSTRUCT_ONLY |
-                          G_PARAM_STATIC_STRINGS));
+        g_object_class_install_property (
+                object_class,
+                PROP_PORT,
+                g_param_spec_uint ("port",
+                                   "M-SEARCH port",
+                                   "UDP port to use for M-SEARCH requests",
+                                   0,
+                                   G_MAXUINT16,
+                                   0,
+                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                           G_PARAM_STATIC_STRINGS));
         /**
          * GSSDPClient:address-family:(attributes org.gtk.Property.get=gssdp_client_get_family):
          *
@@ -650,23 +659,22 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          * used to determine the proper address.
          *
          * If not specified, will contain the currrent address family after
-         * the call to g_initable_init()<!-- -->. Use %G_SOCKET_FAMILY_INVALID
+         * the call to [method@Glib.Initable.init]. Use %G_SOCKET_FAMILY_INVALID
          * to specifiy using the default socket family (legacy IP)
          *
          * Since: 1.2.0
          */
-        g_object_class_install_property
-                (object_class,
-                 PROP_ADDRESS_FAMILY,
-                 g_param_spec_enum
-                        ("address-family",
-                         "IP Address family",
-                         "IP address family to prefer when creating the client",
-                         G_TYPE_SOCKET_FAMILY,
-                         G_SOCKET_FAMILY_INVALID,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS));
+        g_object_class_install_property (
+                object_class,
+                PROP_ADDRESS_FAMILY,
+                g_param_spec_enum (
+                        "address-family",
+                        "IP Address family",
+                        "IP address family to prefer when creating the client",
+                        G_TYPE_SOCKET_FAMILY,
+                        G_SOCKET_FAMILY_INVALID,
+                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:uda-version:(attributes org.gtk.Property.get=gssdp_client_get_uda_version):
@@ -675,18 +683,17 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          *
          * Since: 1.2.0
          */
-        g_object_class_install_property
-                (object_class,
-                 PROP_UDA_VERSION,
-                 g_param_spec_enum
-                        ("uda-version",
-                         "UDA version",
-                         "UPnP Device Architecture version on this client",
-                         GSSDP_TYPE_UDA_VERSION,
-                         GSSDP_UDA_VERSION_1_0,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS));
+        g_object_class_install_property (
+                object_class,
+                PROP_UDA_VERSION,
+                g_param_spec_enum (
+                        "uda-version",
+                        "UDA version",
+                        "UPnP Device Architecture version on this client",
+                        GSSDP_TYPE_UDA_VERSION,
+                        GSSDP_UDA_VERSION_1_0,
+                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+                                G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:boot-id:(attributes org.gtk.Property.set=gssdp_client_set_boot_id):
@@ -695,19 +702,17 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          *
          * Since 1.2.0
          */
-        g_object_class_install_property
-                (object_class,
-                 PROP_BOOT_ID,
-                 g_param_spec_int
-                        ("boot-id",
-                         "current boot-id value",
-                         "Value of the BOOTID.UPNP.ORG header",
-                         -1,
-                         G_MAXINT32,
-                         -1,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT |
-                         G_PARAM_STATIC_STRINGS));
+        g_object_class_install_property (
+                object_class,
+                PROP_BOOT_ID,
+                g_param_spec_int ("boot-id",
+                                  "current boot-id value",
+                                  "Value of the BOOTID.UPNP.ORG header",
+                                  -1,
+                                  G_MAXINT32,
+                                  -1,
+                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
+                                          G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient:config-id:(attributes org.gtk.Property.set=gssdp_client_set_config_id):
@@ -716,19 +721,17 @@ gssdp_client_class_init (GSSDPClientClass *klass)
          *
          * Since 1.2.0
          */
-        g_object_class_install_property
-                (object_class,
-                 PROP_CONFIG_ID,
-                 g_param_spec_int
-                        ("config-id",
-                         "current config-id value",
-                         "Value of the CONFIGID.UPNP.ORG header",
-                         -1,
-                         G_MAXINT32,
-                         -1,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT |
-                         G_PARAM_STATIC_STRINGS));
+        g_object_class_install_property (
+                object_class,
+                PROP_CONFIG_ID,
+                g_param_spec_int ("config-id",
+                                  "current config-id value",
+                                  "Value of the CONFIGID.UPNP.ORG header",
+                                  -1,
+                                  G_MAXINT32,
+                                  -1,
+                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
+                                          G_PARAM_STATIC_STRINGS));
 
         /**
          * GSSDPClient::message-received: (skip)
@@ -759,13 +762,17 @@ gssdp_client_class_init (GSSDPClientClass *klass)
 
 /**
  * gssdp_client_new:
- * @iface: (nullable): The name of the network interface, or %NULL for auto-detection.
+ * @iface: (nullable): The name of the network interface, or %NULL for
+ * auto-detection.
  * @error: (nullable): Location to store error, or %NULL
  *
  * Creates a GSSDP client on @iface. GSSDPClient will pick the address it finds
  * suitable for using.
  *
- * Using this utility function, the created client will be using UDA 1.0 and IPv4 only.
+ * Using this utility function, the created client will be using UDA 1.0 and
+ * IPv4 only.
+ *
+ * Deprecated: 1.6. Use [ctor@GSSDP.Client.new_for_address] instead.
  *
  * Return value: (nullable): A new #GSSDPClient object.
  **/
@@ -792,6 +799,8 @@ gssdp_client_new (const char *iface, GError **error)
  *
  * Using this utility function, the created client will be using UDA 1.0 and IPv4 only.
  *
+ * Deprecated: 1.6. Use [ctor@GSSDP.Client.new_for_address] instead.
+ *
  * Return value: (nullable):  A new #GSSDPClient object or %NULL on error.
  */
 GSSDPClient *
@@ -802,11 +811,78 @@ gssdp_client_new_with_port (const char *iface,
         return g_initable_new (GSSDP_TYPE_CLIENT,
                                NULL,
                                error,
-                               "interface", iface,
-                               "msearch-port", msearch_port,
+                               "interface",
+                               iface,
+                               "port",
+                               msearch_port,
                                NULL);
 }
 
+/**
+ * gssdp_client_new_full:
+ * @iface: (nullable): the name of a network interface
+ * @addr: (nullable): an IP address or %NULL for auto-detection. If you do not
+ * care about the address, but want to specify an address family, use
+ * [ctor@Glib.InetAddress.new_any] with the appropriate family instead.
+ * @port: The network port to use for M-SEARCH requests or 0 for
+ * random.
+ * @uda_version: The UDA version this client will adhere to
+ * @error: (allow-none): Location to store error, or %NULL.
+ *
+ * Creates a GSSDP client with address @addr. If none is specified, GSSDP
+ * will chose the address it deems most suitable.
+ *
+ * Since: 1.6.
+ *
+ * Return value: (nullable):  A new #GSSDPClient object or %NULL on error.
+ */
+GSSDPClient *
+gssdp_client_new_full (const char *iface,
+                       GInetAddress *addr,
+                       guint16 port,
+                       GSSDPUDAVersion uda_version,
+                       GError **error)
+{
+        return g_initable_new (GSSDP_TYPE_CLIENT,
+                               NULL,
+                               error,
+                               "interface",
+                               iface,
+                               "address",
+                               addr,
+                               "port",
+                               port,
+                               "uda-version",
+                               uda_version,
+                               NULL);
+}
+
+/**
+ * gssdp_client_new_for_address
+ * @addr: (nullable): an IP address or %NULL for auto-detection. If you do not
+ * care about the address, but want to specify an address family, use
+ * [ctor@Glib.InetAddress.new_any] with the appropriate family instead.
+ * @port: The network port to use for M-SEARCH requests or 0 for
+ * random.
+ * @uda_version: The UDA version this client will adhere to
+ * @error: (allow-none): Location to store error, or %NULL.
+ *
+ * Creates a GSSDP client with address @addr. If none is specified, GSSDP
+ * will chose the address it deems most suitable.
+ *
+ * Since: 1.6.
+ *
+ * Return value: (nullable):  A new #GSSDPClient object or %NULL on error.
+ */
+GSSDPClient *
+gssdp_client_new_for_address (GInetAddress *addr,
+                              guint16 port,
+                              GSSDPUDAVersion uda_version,
+                              GError **error)
+{
+        return gssdp_client_new_full (NULL, addr, port, uda_version, error);
+}
+
 /**
  * gssdp_client_set_server_id:(attributes org.gtk.Method.set_property=server-id):
  * @client: A #GSSDPClient
@@ -1156,7 +1232,7 @@ gssdp_client_clear_headers (GSSDPClient *client)
 }
 
 /**
- * gssdp_client_get_address:
+ * gssdp_client_get_address:(attributes org.gtk.Method.get_property=address):
  * @client: A #GSSDPClient
  *
  * The IP address this client works on.
@@ -1421,8 +1497,11 @@ _gssdp_client_get_mcast_group (GSSDPClient *client)
                  * address to use the proper multicast group */
                 if (g_inet_address_get_is_link_local (priv->device.host_addr)) {
                             return SSDP_V6_LL;
+                } else if (g_inet_address_get_is_site_local (
+                                   priv->device.host_addr)) {
+                        return SSDP_V6_SL;
                 } else {
-                            return SSDP_V6_SL;
+                        return SSDP_V6_GL;
                 }
         }
 }
@@ -1457,10 +1536,15 @@ _gssdp_client_get_mcast_group_addr (GSSDPClient *client)
                             ENSURE_V6_GROUP(LL);
 
                             return SSDP_V6_LL_ADDR;
-                } else {
-                            ENSURE_V6_GROUP(SL);
+                } else if (g_inet_address_get_is_site_local (
+                                   priv->device.host_addr)) {
+                        ENSURE_V6_GROUP (SL);
 
-                            return SSDP_V6_SL_ADDR;
+                        return SSDP_V6_SL_ADDR;
+                } else {
+                        ENSURE_V6_GROUP (GL);
+
+                        return SSDP_V6_GL_ADDR;
                 }
         }
 }
@@ -1874,17 +1958,53 @@ static gboolean
 init_network_info (GSSDPClient *client, GError **error)
 {
         GSSDPClientPrivate *priv = gssdp_client_get_instance_private (client);
-        gboolean ret = TRUE;
+
+        /* If we were constructed with a host_ip, try to parse a host_addr from that.
+         * Further code will only work with host_addr */
+        if (priv->device.host_ip != NULL) {
+                GInetAddress *addr =
+                        g_inet_address_new_from_string (priv->device.host_ip);
+                if (addr == NULL) {
+                        g_set_error (error,
+                                     GSSDP_ERROR,
+                                     GSSDP_ERROR_FAILED,
+                                     "Unparseable host_ip %s",
+                                     priv->device.host_ip);
+
+                        return FALSE;
+                }
+
+                // If there was also a host address passed (why?!) make sure
+                // they match up, otherwise exit with error as well
+                if (priv->device.host_addr != NULL) {
+                        gboolean equal =
+                                g_inet_address_equal (priv->device.host_addr,
+                                                      addr);
+                        g_object_unref (addr);
+                        if (!equal) {
+                                g_set_error_literal (
+                                        error,
+                                        GSSDP_ERROR,
+                                        GSSDP_ERROR_FAILED,
+                                        "host_ip and host_addr do not match");
+                                return FALSE;
+                        }
+
+                } else {
+                        priv->device.host_addr = addr;
+                }
+
+        }
 
         /* Either interface name or host_ip wasn't given during construction.
          * If one is given, try to find the other, otherwise just pick an
          * interface.
          */
-        if (priv->device.iface_name == NULL ||
-            priv->device.host_addr == NULL  ||
-            priv->device.host_mask == NULL)
-                gssdp_net_get_host_ip (&(priv->device));
-        else {
+        if (priv->device.iface_name == NULL || priv->device.host_addr == NULL ||
+            priv->device.host_mask == NULL) {
+                if (!gssdp_net_get_host_ip (&(priv->device), error))
+                        return FALSE;
+        } else {
                 /* Ugly. Ideally, get_host_ip needs to be run everytime, but
                  * it is currently to stupid so just query index here if we
                  * have a name and an interface already.
@@ -1905,7 +2025,7 @@ init_network_info (GSSDPClient *client, GError **error)
                                      GSSDP_ERROR_FAILED,
                                      "No default route?");
 
-                ret = FALSE;
+                return FALSE;
         } else if (priv->device.host_addr == NULL) {
                 g_set_error (error,
                              GSSDP_ERROR,
@@ -1913,13 +2033,13 @@ init_network_info (GSSDPClient *client, GError **error)
                              "Failed to find IP of interface %s",
                              priv->device.iface_name);
 
-                ret = FALSE;
+                return FALSE;
         } else if (priv->device.host_mask == NULL) {
                 g_set_error (error,
                              GSSDP_ERROR,
                              GSSDP_ERROR_FAILED,
                              "No network mask?");
-                ret = FALSE;
+                return FALSE;
         }
 
         g_debug ("Created SSDP client %p", client);
@@ -1931,5 +2051,5 @@ init_network_info (GSSDPClient *client, GError **error)
         g_debug ("  host_addr  : %p", priv->device.host_addr);
         g_debug ("  host_mask  : %p", priv->device.host_mask);
 
-        return ret;
+        return TRUE;
 }
